@@ -3,11 +3,12 @@ from babel import Locale
 
 
 # TODO: Figure out a nicer way to get this. From the .ini? 
-_KNOWN_LOCALES = ['en', 'ca', 'de', 'el',
-                  'es', 'fi', 'fr', 'hu',
-                  'it', 'lt', 'nl', 'no',
-                  'pl', 'pt_BR', 'ro', 'ru',
-                  'sl', 'sq']
+_KNOWN_LOCALES = ['en', 'de',
+                  'es', 'fi', 'fr', 
+                  'it', 'nl', 'no',
+                  'pt_BR', 'ru',
+                  'sq', 'cs_CZ', 'bg',
+                  ]
 
 def get_available_locales():
     return map(Locale.parse, _KNOWN_LOCALES)
@@ -18,7 +19,8 @@ def get_default_locale():
             Locale.parse('en')
 
 def set_session_locale(locale):
-    assert locale in _KNOWN_LOCALES
+    if locale not in _KNOWN_LOCALES:
+        raise ValueError
     from pylons import session
     session['locale'] = locale
     session.save()
