@@ -7,11 +7,11 @@ from ckan.lib.dictization import (obj_list_dictize,
                                   table_dictize)
 from ckan.logic import NotFound
 import ckan.misc
-import json
+from ckan.lib.helpers import json
 
 ## package save
 
-def group_list_dictize(obj_list, context, sort_key=lambda x:x):
+def group_list_dictize(obj_list, context, sort_key=lambda x:x['display_name']):
 
     active = context.get('active', True)
 
@@ -23,6 +23,10 @@ def group_list_dictize(obj_list, context, sort_key=lambda x:x):
         if active and obj.state not in ('active', 'pending'):
             continue
 
+        group_dict['display_name'] = obj.display_name
+
+        group_dict['packages'] = len(obj.packages)
+
         result_list.append(group_dict)
     return sorted(result_list, key=sort_key)
 
@@ -74,7 +78,11 @@ def resource_dictize(res, context):
     return resource
 
 def _execute_with_revision(q, rev_table, context):
+    '''
+    Raises NotFound if the context['revision_id'] does not exist.
+    Returns [] if there are no results.
 
+    '''
     model = context['model']
     meta = model.meta
     session = model.Session
@@ -83,8 +91,11 @@ def _execute_with_revision(q, rev_table, context):
     pending = context.get('pending')
 
     if revision_id:
-        revision_date = session.query(context['model'].Revision).filter_by(
-            id=revision_id).one().timestamp
+        revision = session.query(context['model'].Revision).filter_by(
+            id=revision_id).first()
+        if not revision:
+            raise NotFound
+        revision_date = revision.timestamp
     
     if revision_date:
         q = q.where(rev_table.c.revision_timestamp <= revision_date)
@@ -148,15 +159,37 @@ def package_dictize(pkg, context):
 def group_dictize(group, context):
 
     result_dict = table_dictize(group, context)
+    
+    result_dict['display_name'] = group.display_name
 
-    result_dict["extras"] = extras_dict_dictize(
+    result_dict['extras'] = extras_dict_dictize(
         group._extras, context)
 
-    result_dict["packages"] = obj_list_dictize(
+    result_dict['packages'] = obj_list_dictize(
         group.packages, context)
 
     return result_dict
 
+def tag_dictize(tag, context):
+
+    result_dict = table_dictize(tag, context)
+
+    result_dict["packages"] = obj_list_dictize(
+        tag.packages_ordered, context)
+    
+    return result_dict 
+
+def user_dictize(user, context):
+
+    result_dict = table_dictize(user, context)
+
+    del result_dict['password']
+    
+    result_dict['display_name'] = user.display_name
+    result_dict['number_of_edits'] = user.number_of_edits()
+    result_dict['number_administered_packages'] = user.number_administered_packages()
+
+    return result_dict 
 
 ## conversion to api
 
@@ -176,6 +209,15 @@ def group_to_api2(group, context):
     dictized["packages"] = sorted([package["id"] for package in dictized["packages"]])
     return dictized
 
+def tag_to_api1(tag, context):
+    
+    dictized = tag_dictize(tag, context)
+    return sorted([package["name"] for package in dictized["packages"]])
+
+def tag_to_api2(tag, context):
+
+    dictized = tag_dictize(tag, context)
+    return sorted([package["id"] for package in dictized["packages"]])
 
 def resource_dict_to_api(res_dict, package_id, context):
     res_dict.pop("revision_id")
