@@ -1,3 +1,4 @@
+from __future__ import with_statement # necessary for python 2.5 support
 import warnings
 from pylons import config
 from sqlalchemy import MetaData, __version__ as sqav
@@ -22,7 +23,7 @@ from rating import *
 from package_relationship import *
 from changeset import Changeset, Change, Changemask
 import ckan.migration
-from ckan.lib.helpers import OrderedDict
+from ckan.lib.helpers import OrderedDict, datetime_to_date_str
 
 # set up in init_model after metadata is bound
 version_table = None
@@ -220,20 +221,43 @@ Revision.groups = property(_get_groups)
 Revision.user = property(_get_revision_user)
 
 def strptimestamp(s):
+    '''Convert a string of an ISO date into a datetime.datetime object.
+    
+    raises TypeError if the number of numbers in the string is not between 3
+                     and 7 (see datetime constructor).
+    raises ValueError if any of the numbers are out of range.
+    '''
+    # TODO: METHOD DEPRECATED - use ckan.lib.helpers.date_str_to_datetime
     import datetime, re
     return datetime.datetime(*map(int, re.split('[^\d]', s)))
 
 def strftimestamp(t):
+    '''Takes a datetime.datetime and returns it as an ISO string. For
+    a pretty printed string, use ckan.lib.helpers.render_datetime.
+    '''
+    # TODO: METHOD DEPRECATED - use ckan.lib.helpers.datetime_to_date_str
     return t.isoformat()
 
-def revision_as_dict(revision, include_packages=True, ref_package_by='name'):
+def revision_as_dict(revision, include_packages=True, include_groups=True,ref_package_by='name'):
     revision_dict = OrderedDict((
         ('id', revision.id),
         ('timestamp', strftimestamp(revision.timestamp)),
         ('message', revision.message),
         ('author', revision.author),
+        ('approved_timestamp',
+         datetime_to_date_str(revision.approved_timestamp) \
+         if revision.approved_timestamp else None),
         ))
     if include_packages:
         revision_dict['packages'] = [getattr(pkg, ref_package_by) \
                                      for pkg in revision.packages]
+    if include_groups:
+        revision_dict['groups'] = [getattr(grp, ref_package_by) \
+                                     for grp in revision.groups]
+       
     return revision_dict
+
+def is_id(id_string):
+    '''Tells the client if the string looks like a revision id or not'''
+    import re
+    return bool(re.match('^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$', id_string))
