@@ -55,7 +55,7 @@ from meta import *
 from vdm.sqlalchemy import StatefulObjectMixin
 from ckan.model.core import DomainObject, State
 from ckan.model import Session, Revision, Package
-from ckan.model import PackageResource, Tag, Group
+from ckan.model import Resource, Tag, Group
 from ckan.model import setup_default_user_roles
 from ckan.lib.helpers import json
 import datetime
@@ -595,7 +595,7 @@ class Sequence(object):
 class Json(object):
     """Dumps and loads JSON strings into Python objects."""
 
-    def dumps(self, data):
+    def dumps(cls, data):
         try:
             json_str = json.dumps(data, indent=2)
         except Exception, inst:
@@ -605,7 +605,7 @@ class Json(object):
 
     dumps = classmethod(dumps)
 
-    def loads(self, json_str):
+    def loads(cls, json_str):
         try:
             data = json.loads(json_str)
         except Exception, inst:
@@ -981,7 +981,7 @@ class TrackedObjectRegister(ObjectRegister):
 
     def get_columns(self):
         """Returns the model of the entity attributes."""
-        from ckan.model.core import orm
+        from sqlalchemy import orm
         table = orm.class_mapper(self.object_type).mapped_table
         return table.c
 
@@ -1632,7 +1632,7 @@ class PackageRegister(TrackedObjectRegister):
         if 'resources' in vector.new:
             entity.resources = []
             for resource_data in vector.new['resources']:
-                package_resource = PackageResource(
+                package_resource = Resource(
                     url=resource_data.get('url', u''),
                     format=resource_data.get('format', u''),
                     description=resource_data.get('description', u''),
