@@ -28,7 +28,9 @@ from paste.deploy import loadapp
 from ckan.lib.create_test_data import CreateTestData
 from ckan.lib import search
 from ckan.lib.helpers import _flash, url_for
+from ckan.lib.helpers import json
 import ckan.model as model
+from ckan import ckan_nose_plugin
 
 __all__ = ['url_for',
            'TestController',
@@ -39,6 +41,7 @@ __all__ = ['url_for',
            'CommonFixtureMethods',
            'TestCase',
            'SkipTest',
+           'CkanServerCase',
         ]
 
 here_dir = os.path.dirname(os.path.abspath(__file__))
@@ -47,6 +50,18 @@ conf_dir = os.path.dirname(os.path.dirname(here_dir))
 # Invoke websetup with the current config file
 SetupCommand('setup-app').run([config['__file__']])
 
+# monkey patch paste.fixtures.TestRespose
+# webtest (successor library) already has this
+# http://pythonpaste.org/webtest/#parsing-the-body
+def _getjson(self):
+    return json.loads(self.body)
+paste.fixture.TestResponse.json = property(_getjson)
+
+# Check config is correct for sqlite
+if config['sqlalchemy.url'].startswith('sqlite:'):
+    assert ckan_nose_plugin.CkanNose.settings.is_ckan, \
+           'You forgot the "--ckan" nosetest setting - see README.txt'
+
 class BaseCase(object):
 
     def setup(self):
@@ -163,13 +178,8 @@ class CommonFixtureMethods(BaseCase):
         return model.User.by_name(name)
 
     @staticmethod
-    def get_harvest_source_by_url(source_url, default=Exception):
-        return model.HarvestSource.get(source_url, default, 'url')
-
-    def create_harvest_source(self, **kwds):
-        source = model.HarvestSource(**kwds)
-        source.save()
-        return source
+    def get_tag_by_name(name):
+        return model.Tag.by_name(name)
 
     def purge_package_by_name(self, package_name):
         package = self.get_package_by_name(package_name)
@@ -206,6 +216,21 @@ class CommonFixtureMethods(BaseCase):
     def anna(self):
         return self.get_package_by_name(u'annakarenina')
 
+    @property
+    def roger(self):
+        return self.get_group_by_name(u'roger')
+
+    @property
+    def david(self):
+        return self.get_group_by_name(u'david')
+
+    @property
+    def russian(self):
+        return self.get_tag_by_name(u'russian')
+
+    @property
+    def tolstoy(self):
+        return self.get_tag_by_name(u'tolstoy')
 
 class CheckMethods(BaseCase):
 
@@ -336,6 +361,11 @@ class TestSearchIndexer:
     def index(cls):
         pass     
 
+    @classmethod
+    def list(cls):
+        return [model.Package.get(pkg_index.package_id).name for pkg_index in model.Session.query(model.PackageSearch)]
+            
+
 def is_search_supported():
     supported_db = "sqlite" not in config.get('sqlalchemy.url')
     return supported_db
@@ -353,14 +383,14 @@ def search_related(test):
         raise SkipTest("Search not supported")
     if not is_search_supported():
         return make_decorator(test)(skip_test)
-    return make_decorator(test)
-    
+    return test
+
 def regex_related(test):
     def skip_test(*args):
         raise SkipTest("Regex not supported")
     if not is_regex_supported():
         return make_decorator(test)(skip_test)
-    return make_decorator(test)
+    return test
 
 def clear_flash(res=None):
     messages = _flash.pop_messages()
